import pytest
import os
from llmx import llm
from llmx.datamodel import TextGenerationConfig


config = TextGenerationConfig(
    n=2,
    temperature=0.4,
    max_tokens=100,
    top_p=1.0,
    top_k=50,
    frequency_penalty=0.0,
    presence_penalty=0.0,
    use_cache=False
)

messages = [
    {"role": "user",
     "content": "What is the capital of France? Only respond with the exact answer"}]


def test_openai():
    openai_gen = llm(provider="openai")
    openai_response = openai_gen.generate(messages, config=config)
    answer = openai_response.text[0].content
    print(openai_response.text[0].content)

    assert ("paris" in answer.lower())
    assert len(openai_response.text) == 2


def test_google():
    google_gen = llm(provider="google")
    config.model = "chat-bison@001"
    google_response = google_gen.generate(messages, config=config)
    answer = google_response.text[0].content
    print(google_response.text[0].content)

    assert ("paris" in answer.lower())
    # assert len(google_response.text) == 2 palm may chose to return 1 or 2 responses


def test_cohere():
    cohere_gen = llm(provider="cohere")
    config.model = "command"
    cohere_response = cohere_gen.generate(messages, config=config)
    answer = cohere_response.text[0].content
    print(cohere_response.text[0].content)

    assert ("paris" in answer.lower())
    assert len(cohere_response.text) == 2


@pytest.mark.skipif(os.environ.get("LLMX_RUNALL", None) is None
                    or os.environ.get("LLMX_RUNALL", None) == "False", reason="takes too long")
def test_hf_local():
    hf_local_gen = llm(
        provider="hf",
        model="TheBloke/Llama-2-7b-chat-fp16",
        device_map="auto")
    hf_local_response = hf_local_gen.generate(messages, config=config)
    answer = hf_local_response.text[0].content
    print(hf_local_response.text[0].content)

    assert ("paris" in answer.lower())
    assert len(hf_local_response.text) == 2



import typer
import os
from typing_extensions import Annotated
from .generators.text.providers import providers

app = typer.Typer()


@app.command()
def models():
    print("Available models:")
    for provider in providers.items():
        print(f"Provider: {provider[1]['name']}")
        for model in provider[1]["models"]:
            print(f"  - {model['name']}")


@app.command()
def list():
    print("list")


def run():
    app()


if __name__ == "__main__":
    app()



from .openai_textgen import OpenAITextGenerator
from .palm_textgen import PalmTextGenerator
from .cohere_textgen import CohereTextGenerator


def llm(provider: str = "openai", **kwargs):
    if provider.lower() == "openai" or provider.lower() == "default":
        return OpenAITextGenerator(**kwargs)
    elif provider.lower() == "palm" or provider.lower() == "google":
        return PalmTextGenerator(provider=provider, **kwargs)
    elif provider.lower() == "cohere":
        return CohereTextGenerator(provider=provider, **kwargs)
    elif provider.lower() == "hf" or provider.lower() == "huggingface":
        try:
            import transformers
            from transformers import (
                AutoTokenizer,
                AutoModelForCausalLM,
                GenerationConfig,
            )
        except ImportError:
            raise ImportError(
                "Please install the `transformers` package to use the HFTextGenerator class. pip install llmx[transformers]"
            )

        # Check if torch package is installed
        try:
            import torch
        except ImportError:
            raise ImportError(
                "Please install the `torch` package to use the HFTextGenerator class.  pip install llmx[transformers]"
            )

        from .hf_textgen import HFTextGenerator

        return HFTextGenerator(provider=provider, **kwargs)

    else:
        raise ValueError(
            f"Invalid provider '{provider}'.  Supported providers are 'openai', 'hf', 'palm', and 'cohere'."
        )



from dataclasses import asdict
from typing import Any, Optional, Union, List
from pydantic.dataclasses import dataclass


@dataclass
class Message:
    role: str
    content: str

    def __post_init__(self):
        self._fields_dict = asdict(self)

    def __getitem__(self, key: Union[str, int]) -> Any:
        return self._fields_dict.get(key)

    def to_dict(self):
        return self._fields_dict


@dataclass
class TextGenerationConfig:
    n: int = 1
    temperature: float = 0.1
    max_tokens: Union[int, None] = None
    top_p: float = 1.0
    top_k: int = 50
    frequency_penalty: float = 0.0
    presence_penalty: float = 0.0
    provider: Union[str, None] = None
    model: Optional[str] = None
    stop: Union[List[str], str, None] = None
    use_cache: bool = True

    def __post_init__(self):
        self._fields_dict = asdict(self)

    def __getitem__(self, key: Union[str, int]) -> Any:
        return self._fields_dict.get(key)


@dataclass
class TextGenerationResponse:
    """Response from a text generation"""

    text: List[Message]
    config: Any
    logprobs: Optional[Any] = None
    usage: Optional[Any] = None

    def __post_init__(self):
        self._fields_dict = asdict(self)

    def __getitem__(self, key: Union[str, int]) -> Any:
        return self._fields_dict.get(key)
